@@ -56,6 +56,12 @@ extern int percentage_hide;
 extern int8_t battery_low_status;
 extern char *battery_low_cmd;
 
+extern char *battery_lclick_command;
+extern char *battery_mclick_command;
+extern char *battery_rclick_command;
+extern char *battery_uwheel_command;
+extern char *battery_dwheel_command;
+
 // default global data
 void default_battery();
 
@@ -71,4 +77,6 @@ void draw_battery(void *obj, cairo_t *c);
 
 int  resize_battery(void *obj);
 
+void battery_action(int button);
+
 #endif
