@@ -796,6 +796,22 @@ int click_clock(Panel *panel, int x, int y)
 }
 
 
+#ifdef ENABLE_BATTERY
+int click_battery(Panel *panel, int x, int y)
+{
+	Battery bat = panel->battery;
+	if (panel_horizontal) {
+		if (bat.area.on_screen && x >= bat.area.posx && x <= (bat.area.posx + bat.area.width))
+			return TRUE;
+	} else {
+		if (bat.area.on_screen && y >= bat.area.posy && y <= (bat.area.posy + bat.area.height))
+			return TRUE;
+	}
+	return FALSE;
+}
+#endif
+
+
 Area* click_area(Panel *panel, int x, int y)
 {
 	Area* result = &panel->area;
